//////////////////////////////////////////////////////////////////////////
//	"fcvtest.h"
#ifndef	FCV_H_INCLUDED
#define	FCV_H_INCLUDED

#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<time.h>
#include	<tchar.h>

#include	<list>
#include	<windows.h>
#include	"mt19937.h"
#include	"crc32.h"

#if	defined(WIN32)
typedef __int64				FileSize;
typedef FILETIME			FileTime;
typedef DWORD				FileAttr;
typedef TCHAR				FileChar;
#else
# error
#endif

class	RandLib : public MersenneTwister
{
public:
	int		rand(void) { return (int) this->genrand_int32(); }
};

class	FileData;
class	FileList;
class	FcvTest;
class	FWrite;

class	FileData
{
	friend class	FileList;

public:
	FileChar		*filename;
	FileSize		filesize;
	FileAttr		fileattr;
	FileTime		timestamp;
	crc32_t			crc;

	FileList		*filelist;

public:
	FileData();
	virtual ~FileData();
	int		SetFilename(const FileChar * name);
};

class	FileList
{
public:
	typedef std::list<FileData *>	List;
	typedef List::iterator			iterator;

	enum filelist_enum
	{
		TOTAL_FILESIZE   = 100 * 1024 * 1024,
		MIN_FILESIZE     = 1,
		MAX_FILESIZE     = 100 * 1024,
		MAX_FILE_NUM     = 2000,
		MAX_DIR_FILE_NUM =  100,
		MAX_DIR_LEVEL    =    4,
	};
	struct ListCreateParam
	{
		const FileChar		*dirname;
		const FileChar		*filename;
		FileSize			totalFileSize;		// t@CTCY
		FileSize			minFileSize;		// ŏt@CTCY
		FileSize			maxFileSize;		// őt@CTCY
		unsigned			maxFileNum;			// t@C
		unsigned			maxDirFileNum;		// fBNg̍őt@C
		unsigned			maxDirLevel;		// őfBNgx
		ListCreateParam()
		{
			this->filename = TEXT("file-");
			this->dirname  = TEXT("DIR-");
			this->totalFileSize = TOTAL_FILESIZE;
			this->minFileSize   = MIN_FILESIZE;
			this->maxFileSize   = MAX_FILESIZE;
			this->maxFileNum    = MAX_FILE_NUM;
			this->maxDirFileNum = MAX_DIR_FILE_NUM;
			this->maxDirLevel   = MAX_DIR_LEVEL;
		}
	};

private:
	List			list;
	unsigned		fileNum;
	unsigned		dirNum;
	static RandLib	rand;

public:
	FileList();
	~FileList();

	void		clear(void);
	unsigned	size(void) { return (unsigned)this->list.size(); }
	iterator	begin(void){ return this->list.begin(); }
	iterator	end(void){ return this->list.end(); }
	void		push_back(FileData * fdata);

	unsigned	GetFileNum(void) const { return this->fileNum; }
	unsigned	GetDirNum(void) const { return this->dirNum; }

	int					ReadDir(const FileChar * dir, BOOL subdir);
	static FileChar *	AddPath(const FileChar * base, const FileChar * name);
	int					CreateFileList(ListCreateParam & lcp);

};

class	FcvTest
{
public:
	enum FcvTest_enum
	{
		BUFFER_SIZE  = 0x20000,
		RANDTBL_SIZE = 0x40000,
	};
	time_t		start_time;
	time_t		finish_time;
	unsigned	bufferSize;
	void		*buffer;

	Crc32		crc;

	RandLib		rand;
	BYTE		*randtbl;
	unsigned	randtblSize;
	unsigned	randtblPos;

	FileList	createlist;
	unsigned	createFiles;
	unsigned	createDirs;
	FileSize	createFileSize;

	unsigned	verifyFiles;
	FileSize	verifyFileSize;
	unsigned	verifyError;

	unsigned	removeFiles;
	unsigned	removeDirs;
	FileSize	removeFileSize;
	unsigned	removeError;

public:
	FcvTest();
	virtual ~FcvTest();
private:
	int			CreateBuffer(unsigned size = BUFFER_SIZE);
	int			CreateRandTbl(unsigned size = RANDTBL_SIZE);
public:
	int			Create(FileList & filelist, const FileChar * dst);
	int			CreateSub(FileList & filelist, const FileChar * dst);
	int			CreateFile(const FileChar * path, FileData & fdata);

	int			Verify(FileList & filelist, const FileChar * dst);
	int			VerifySub(FileList & filelist, const FileChar * dst);
	int			VerifyFile(const FileChar * path, FileData & fdata);

	int			Remove(void);
};

class	Fcv
{
public:
	enum verify_mode_enum
	{
		VERIFY_OFF  = 0,
		VERIFY_FILE = 1,
		VERIFY_ALL  = 2,
	};

	unsigned		verifyMode;
	unsigned		filesize_m;

public:
	Fcv();

	static TCHAR *	GetErrMsg(int errcode);
	static TCHAR *	FormatTime(TCHAR * buffer, time_t tm);
	static TCHAR *	FormatFileSize(TCHAR * buffer, FileSize size);
};

#endif
