//////////////////////////////////////////////////////////////////////////
//	"fcvtest.cpp"
//------------------------------------------------------------------------
//////////////////////////////////////////////////////////////////////////

#include	"fcv.h"

FcvTest::FcvTest()
{
	this->bufferSize = 0;
	this->buffer = NULL;

	this->randtbl = NULL;
	this->randtblSize = 0;
	this->randtblPos = 0;
}

FcvTest::~FcvTest()
{
	if ( this->buffer != NULL )
		delete [] this->buffer;
	if ( this->randtbl != NULL )
		delete [] this->randtbl;
}

int		FcvTest::CreateBuffer(unsigned size)
{
	if ( this->buffer != NULL )
		delete [] this->buffer;
	if ( (this->buffer = new BYTE [size]) == NULL )
		return -1;
	this->bufferSize = size;
	return 0;
}

int		FcvTest::CreateRandTbl(unsigned size)
{
	if ( this->randtbl != NULL )
		delete [] this->randtbl;
	if ( (this->randtbl = new BYTE [size]) == NULL )
		return -1;
	this->randtblSize = size;

	{
		unsigned	i, datasize = this->randtblSize;
		BYTE		*d;

		d = (BYTE *)this->randtbl;
		i = 0;
		while ( i < datasize )
		{
			DWORD	dw;

			dw = (DWORD) this->rand.rand();
			if ( i + sizeof(DWORD) < datasize )
			{
				*(DWORD *) d = dw;
				i += sizeof(DWORD);
				d += sizeof(DWORD);
			} else
			{
				while ( i < datasize )
				{
					*d++ = (BYTE)dw;
					i++;
					dw >>= 8;
				}
			}
		}
	}

	return 0;
}

//////////////////////////////////////////////////////////////////////////
//	Create/Write

int		FcvTest::Create(FileList & filelist, const FileChar * dst)
{
	int			res;
	TCHAR		msg[1024];

	if ( this->buffer == NULL )
	{
		if ( this->CreateBuffer() < 0 )
			return -1;
	}
	if ( this->randtbl == NULL )
	{
		if ( this->CreateRandTbl() < 0 )
			return -1;
	}
	this->createFiles = 0;
	this->createDirs   = 0;
	this->createFileSize = 0;

	::time(&this->start_time);
	_ftprintf(stderr, TEXT("%s Start\n"), Fcv::FormatTime(msg, this->start_time) );
	res = this->CreateSub(filelist, dst);
	::time(&this->finish_time);
	_ftprintf(stderr, TEXT("%s Finish\n"), Fcv::FormatTime(msg, this->finish_time) );
	if ( res < 0 )
	{
		_ftprintf(stderr, TEXT("FcvTest Error!\n"));
		return -1;
	}
	{
		unsigned	dt;

		dt = (unsigned)::difftime(this->finish_time, this->start_time);
		if ( dt == 0 )
			dt = 1;
		_ftprintf(stdout, TEXT("Create %u files, %u directories\n"), this->createFiles, this->createDirs);
		_ftprintf(stdout, TEXT("Create filesize %s bytes\n"), Fcv::FormatFileSize(msg, this->createFileSize) );
		_ftprintf(stdout, TEXT("Create time %u sec\n"), dt );
		_ftprintf(stdout, TEXT("Create speed %u.%uKB/sec (%uB/sec)\n")
		  , (unsigned)(this->createFileSize / 1000 / dt),  (unsigned)(this->createFileSize / 100 / dt) % 10
		  , (unsigned)(this->createFileSize / dt) );
	}

	return 0;
}

int		FcvTest::CreateSub(FileList & filelist, const FileChar * dst)
{
	FileList::iterator	it;

	for ( it = filelist.begin(); it != filelist.end(); it++ )
	{
		FileChar		*path;
		FileData		*fdata;

		fdata = *it;
		if ( (path = FileList::AddPath(dst, fdata->filename)) == NULL )
			return -1;	// error
		{	// Rs[Xgɓo^
			FileData	*fdata2;

			if ( (fdata2 = new FileData) == NULL )
				return -1;
			fdata2->SetFilename(path);
			fdata2->filesize  = fdata->filesize;
			fdata2->fileattr  = fdata->fileattr;
			fdata2->timestamp = fdata->timestamp;
			fdata2->crc       = fdata->crc;
			this->createlist.push_back(fdata2);
		}
		if ( fdata->fileattr & FILE_ATTRIBUTE_DIRECTORY )
		{	// fBNg
			_ftprintf(stderr, TEXT("CreateDirecrory %s\n"), path);
			::CreateDirectory(path, NULL);
			this->createDirs++;
			if ( this->CreateSub(*(fdata->filelist), path) < 0 )
				return -1;
		} else
		{
			{
				TCHAR	tmp[80];

				_ftprintf(stderr, TEXT("CreateFile %s (%s bytes) No.%u\n")
				  , path
				  , Fcv::FormatFileSize(tmp, fdata->filesize)
				  , this->createFiles + 1 );
			}
			if ( this->CreateFile(path, *fdata) < 0 )
				return -1;
		}
		delete [] path;
	}
	return 0;
}


int		FcvTest::CreateFile(const FileChar * path, FileData & fdata)
{
	HANDLE		hFile;
	FileSize	remainSize;

	hFile = ::CreateFile( path
	  , GENERIC_WRITE
	  , FILE_SHARE_READ | FILE_SHARE_WRITE
	  , NULL
	  , CREATE_ALWAYS
	  , FILE_ATTRIBUTE_NORMAL
	  , NULL
	);
	if ( hFile == INVALID_HANDLE_VALUE )
	{
		_ftprintf(stderr, TEXT("CreateFile Error!: %s\n"), Fcv::GetErrMsg(::GetLastError()) );
		return -1;
	}
	this->crc.Init();
	remainSize = fdata.filesize;
	while ( remainSize > 0 )
	{
		unsigned	datasize;

		if ( remainSize > this->bufferSize )
			datasize = this->bufferSize;
		else
			datasize = (unsigned) remainSize;
		{
			unsigned	i;
			BYTE		*d;

			d = (BYTE *)this->buffer;
			for ( i = 0; i < datasize; i++ )
			{
				*d++ = this->randtbl[this->randtblPos];
				this->randtblPos = (this->randtblPos + 1) % this->randtblSize;
			}
		}
		{
			DWORD	dw;

			this->crc.Calc(datasize, (BYTE *)this->buffer);
			if ( ::WriteFile(hFile, this->buffer, (DWORD)datasize, &dw, NULL) == 0 )
			{	// error
				_ftprintf(stderr, TEXT("WriteFile Error!: %s\n"), Fcv::GetErrMsg(::GetLastError()) );
				::CloseHandle(hFile);
				return -1;
			}
			datasize = dw;
		}
		remainSize -= datasize;
	}
	::CloseHandle(hFile);
	fdata.crc = this->crc.GetCrc();
	this->createFiles++;
	this->createFileSize += fdata.filesize;
	return 0;
}

//////////////////////////////////////////////////////////////////////////
//	Verify/Read

int		FcvTest::Verify(FileList & filelist, const FileChar * dst)
{
	int			res;
	TCHAR		msg[1024];

	if ( this->buffer == NULL )
	{
		if ( this->CreateBuffer() < 0 )
			return -1;
	}
	this->verifyFiles = 0;
	this->verifyFileSize = 0;
	this->verifyError = 0;

	::time(&this->start_time);
	_ftprintf(stderr, TEXT("%s Start\n"), Fcv::FormatTime(msg, this->start_time) );
	res = this->VerifySub(filelist, dst);
	::time(&this->finish_time);
	_ftprintf(stderr, TEXT("%s Finish\n"), Fcv::FormatTime(msg, this->finish_time) );
	if ( res < 0 )
	{
		_ftprintf(stderr, TEXT("Verify Error!\n"));
		return -1;
	}
	{
		unsigned	dt;

		dt = (unsigned)::difftime(this->finish_time, this->start_time);
		if ( dt == 0 )
			dt = 1;
		_ftprintf(stdout, TEXT("Verify %u files, %u CRC Errors\n"), this->verifyFiles, this->verifyError);
		_ftprintf(stdout, TEXT("Verify filesize %s bytes\n"), Fcv::FormatFileSize(msg, this->verifyFileSize) );
		_ftprintf(stdout, TEXT("Verify time %u sec\n"), dt );
		_ftprintf(stdout, TEXT("Verify speed %u.%uKB/sec (%uB/sec)\n")
		  , (unsigned)(this->verifyFileSize / 1000 / dt),  (unsigned)(this->verifyFileSize / 100 / dt) % 10
		  , (unsigned)(this->verifyFileSize / dt) );
	}

	return 0;
}

int		FcvTest::VerifySub(FileList & filelist, const FileChar * dst)
{
	FileList::iterator	it;

	for ( it = filelist.begin(); it != filelist.end(); it++ )
	{
		FileChar		*path;
		FileData		*fdata;

		fdata = *it;
		if ( (path = FileList::AddPath(dst, fdata->filename)) == NULL )
			return -1;	// error
		if ( fdata->fileattr & FILE_ATTRIBUTE_DIRECTORY )
		{	// fBNg
			if ( this->VerifySub(*(fdata->filelist), path) < 0 )
				return -1;
		} else
		{
			{
				TCHAR	tmp[80];

				_ftprintf(stderr, TEXT("Verify %s (%08X:%s bytes) No.%u\n")
				  , path
				  , fdata->crc
				  , Fcv::FormatFileSize(tmp, fdata->filesize)
				  , this->verifyFiles + 1 );
			}
			if ( this->VerifyFile(path, *fdata) < 0 )
				return -1;
		}
		delete [] path;
	}
	return 0;
}

int		FcvTest::VerifyFile(const FileChar * path, FileData & fdata)
{
	HANDLE		hFile;
	FileSize	remainSize;

	hFile = ::CreateFile( path
	  , GENERIC_READ
	  , FILE_SHARE_READ
	  , NULL
	  , OPEN_EXISTING
	  , FILE_ATTRIBUTE_NORMAL
	  , NULL
	);
	if ( hFile == INVALID_HANDLE_VALUE )
	{
		_ftprintf(stderr, TEXT("CreateFile Error!: %s\n"), Fcv::GetErrMsg(::GetLastError()) );
		return -1;
	}
	this->crc.Init();
	remainSize = fdata.filesize;
	while ( remainSize > 0 )
	{
		unsigned	datasize;

		if ( remainSize > this->bufferSize )
			datasize = this->bufferSize;
		else
			datasize = (unsigned) remainSize;

		{
			DWORD	dw;

			if ( ::ReadFile(hFile, this->buffer, (DWORD)datasize, &dw, NULL) == 0 )
			{	// error
				_ftprintf(stderr, TEXT("ReadFile Error!: %s\n"), Fcv::GetErrMsg(::GetLastError()) );
				::CloseHandle(hFile);
				return -1;
			}
			datasize = dw;
			if ( datasize > 0 )
				this->crc.Calc(datasize, (BYTE *)this->buffer);
		}
		if ( datasize == 0 )
		{
			_ftprintf(stderr, TEXT("FileSize Error!\n") );
			::CloseHandle(hFile);
			return -1;
		}
		remainSize -= datasize;
	}
	::CloseHandle(hFile);

	this->verifyFiles++;
	this->verifyFileSize += fdata.filesize;
	if ( fdata.crc != this->crc.GetCrc() )
	{
		this->verifyError++;
		_ftprintf(stderr, TEXT("CRC Error!\n") );
	}

	return 0;
}

//////////////////////////////////////////////////////////////////////////
//	Remove

int		FcvTest::Remove(void)
{
	int			res;
	TCHAR		msg[1024];
	FileList	filelist;

	if ( this->createlist.size() == 0 )
	{
		return 0;
	}
	this->removeFiles = 0;
	this->removeDirs  = 0;
	this->removeFileSize = 0;
	this->removeError = 0;

	::time(&this->start_time);
	_ftprintf(stderr, TEXT("%s Start\n"), Fcv::FormatTime(msg, this->start_time) );
	{
		FileList::iterator	it;

		res = 0;
		it = this->createlist.end();
		do
		{
			FileData			*fdata;

			--it;
			fdata = *it;
			if ( fdata->fileattr & FILE_ATTRIBUTE_DIRECTORY )
			{	// fBNg
				_ftprintf(stderr, TEXT("RemoveDirecrory %s\n"), fdata->filename);
				if ( ::RemoveDirectory(fdata->filename) == 0 )
				{
					res = -1;
					break;
				}
				this->removeDirs++;
			} else
			{
				TCHAR	tmp[80];

				_ftprintf(stderr, TEXT("RemoveFile %s (%s bytes) No.%u\n")
				  , fdata->filename
				  , Fcv::FormatFileSize(tmp, fdata->filesize)
				  , this->removeFiles + 1 );
				if ( ::DeleteFile(fdata->filename) == 0 )
				{
					res = -1;
					break;
				}
				this->removeFiles++;
				this->removeFileSize += fdata->filesize;
			}
		} while ( it != this->createlist.begin() );
	}
	::time(&this->finish_time);
	_ftprintf(stderr, TEXT("%s Finish\n"), Fcv::FormatTime(msg, this->finish_time) );
	if ( res < 0 )
	{
		_ftprintf(stderr, TEXT("Remove Error!\n"));
		return -1;
	}
	{
		unsigned	dt;

		dt = (unsigned)::difftime(this->finish_time, this->start_time);
		if ( dt == 0 )
			dt = 1;
		_ftprintf(stdout, TEXT("Remove %u files, %u directories\n"), this->removeFiles, this->removeDirs);
		_ftprintf(stdout, TEXT("Remove filesize %s bytes\n"), Fcv::FormatFileSize(msg, this->removeFileSize) );
		_ftprintf(stdout, TEXT("Remove time %u sec\n"), dt );
		_ftprintf(stdout, TEXT("Remove speed %u.%uKB/sec (%uB/sec)\n")
		  , (unsigned)(this->removeFileSize / 1000 / dt),  (unsigned)(this->removeFileSize / 100 / dt) % 10
		  , (unsigned)(this->removeFileSize / dt) );
	}

	return 0;
}
