//////////////////////////////////////////////////////////////////////////
// A straightforward C++ implementation of MT19937
// Haruhiko Okumura <okumura@matsusaka-u.ac.jp>
// Based on C source by Takuji Nishimura and Makoto Matsumoto
// (initialization improved 2002-01-26)

#include	<stdlib.h>
#include	"mt19937.h"

const int						MersenneTwister::N = 624;
const int						MersenneTwister::M = 397;
const MersenneTwister::uint32	MersenneTwister::MAT_A = 0x9908b0dfUL;
const MersenneTwister::uint32	MersenneTwister::UMASK = 0x80000000UL;
const MersenneTwister::uint32	MersenneTwister::LMASK = 0x7fffffffUL;

void	MersenneTwister::init(uint32 s)
{
	int			i;

	if ( (this->mt = new uint32 [N]) == NULL )
		return;	// error
	this->mti = N;
	this->mt[0] = s & 0xffffffffUL;
	for ( i = 1; i < N; i++)
	{
		this->mt[i] = 1812433253UL * (this->mt[i-1] ^ (this->mt[i-1] >> 30)) + i;
		this->mt[i] &= 0xffffffffUL;
	}
}

MersenneTwister::MersenneTwister(uint32 s)
{
	this->init(s);
}

MersenneTwister::MersenneTwister(uint32 init_key[], int key_length)
{
	int		i = 1;
	int		j = 0;
	int		k;

	this->init(19650218UL);
	for ( k = (N > key_length) ? N : key_length; k != 0; k--)
	{
		this->mt[i] = (this->mt[i] ^ ((this->mt[i-1] ^ (this->mt[i-1] >> 30)) * 1664525UL)) + init_key[j] + j;
		this->mt[i] &= 0xffffffffUL;
		if ( ++i >= N )
		{
			this->mt[0] = this->mt[N-1];
			i = 1;
		}
		if ( ++j >= key_length )
			j = 0;
	}
	for ( k = N - 1; k != 0; k--)
	{
		this->mt[i] = (this->mt[i] ^ ((this->mt[i-1] ^ (this->mt[i-1] >> 30)) * 1566083941UL)) - i;
		this->mt[i] &= 0xffffffffUL;
		if ( ++i >= N )
		{
			this->mt[0] = this->mt[N-1];
			i = 1;
		}
	}
	this->mt[0] = 0x80000000UL;
}

MersenneTwister::~MersenneTwister()
{
	if ( this->mt != NULL )
		delete [] this->mt;
}

MersenneTwister::uint32		MersenneTwister::genrand_int32(void)
{
	if ( this->mti == N )
	{
		int		k;

		for ( k = 0; k < N - M; k++)
			this->mt[k] = this->mt[k+M] ^ this->twist(this->mt[k], this->mt[k+1]);
		for ( k = N - M; k < N - 1; k++)
			mt[k] = mt[k-(N-M)] ^ twist(mt[k], mt[k+1]);
		mt[N-1] = mt[M-1] ^ twist(mt[N-1], mt[0]);
		mti = 0;
	}
	{
		uint32	y;

		y = this->mt[this->mti++];
		y ^= (y >> 11);
		y ^= (y <<  7) & 0x9d2c5680UL;
		y ^= (y << 15) & 0xefc60000UL;
		y ^= (y >> 18);
		return y;
	}
}
